@@ -8,6 +8,7 @@
  * Contributors:
  *   Balasko, Jeno
  *   Raduly, Csaba
+ *   Szabo, Bence Janos
  *
  ******************************************************************************/
 #include "TypeCompat.hh"
@@ -84,128 +85,6 @@ string TypeConv::get_conv_func(Type *p_from, Type *p_to, Module *p_mod)
   return ret_val;
 }
 
-bool TypeConv::needs_conv_redir(TemplateInstance *p_ti, Reference *p_ref)
-{
-  if (!use_runtime_2) FATAL_ERROR("TypeConv::needs_conv_redir()");
-  if (!p_ref)
-    return false;
-  Template *templ_body = p_ti->get_Template();
-  Template::templatetype_t templ_body_tt = templ_body->get_templatetype();
-  switch (templ_body_tt) {
-  case Template::SPECIFIC_VALUE:
-    if (templ_body->get_specific_value()->get_needs_conversion())
-      return true;
-    break;
-  default:
-    // If the first parameter needs conversion, go ahead.  It's most
-    // probably a reference.
-    if (templ_body->get_needs_conversion())
-      return true;
-    break;
-  }
-  // Check if the value and the type specified for the template instance
-  // match.  If not, we'll need a conversion.
-  if (p_ti->get_Type()) {
-    Type *ti_type = p_ti->get_Type()->get_type_refd_last();
-    Type *val_type = p_ref->get_refd_assignment()->get_Type()
-      ->get_field_type(p_ref->get_subrefs(), Type::EXPECTED_DYNAMIC_VALUE)
-      ->get_type_refd_last();
-    if (ti_type->is_structured_type() && val_type->is_structured_type()
-        && ti_type != val_type)
-      return true;
-  }
-  return false;
-}
-
-void TypeConv::gen_conv_code_redir(expression_struct *p_expr,
-                                   TemplateInstance *p_ti, Reference *p_ref)
-{
-  if (!use_runtime_2) FATAL_ERROR("TypeConv::gen_conv_code_redir()");
-  Template *templ_body = p_ti->get_Template();
-  Template::templatetype_t templ_body_tt = templ_body->get_templatetype();
-  expression_struct expr_tmp;
-  Code::init_expr(&expr_tmp);
-  const string& tmp_id1 = templ_body->get_temporary_id();
-  const char *tmp_id_str1 = tmp_id1.c_str();
-  const string& tmp_id2 = templ_body->get_temporary_id();
-  const char *tmp_id_str2 = tmp_id2.c_str();
-  const string& tmp_id3 = templ_body->get_temporary_id();
-  const char *tmp_id_str3 = tmp_id3.c_str();
-  Type *my_gov = p_ref->get_refd_assignment()->get_Type()
-    ->get_field_type(p_ref->get_subrefs(), Type::EXPECTED_DYNAMIC_VALUE)
-    ->get_type_refd_last();
-  Type *orig_type = NULL;
-  switch (templ_body_tt) {
-  case Template::SPECIFIC_VALUE: {
-    Value *val = templ_body->get_specific_value();
-    if (val->get_valuetype() == Value::V_REFD)
-      orig_type = val->get_reference()->get_refd_assignment()->get_Type()
-        ->get_field_type(val->get_reference()->get_subrefs(),
-        Type::EXPECTED_DYNAMIC_VALUE)->get_type_refd_last();
-    break; }
-  case Template::TEMPLATE_REFD: {
-    Reference *ref = templ_body->get_reference();
-    if (!ref) FATAL_ERROR("TypeConv::gen_conv_code_redir()");
-    orig_type = ref->get_refd_assignment()->get_Type()->get_field_type(ref
-      ->get_subrefs(), Type::EXPECTED_TEMPLATE)->get_type_refd_last();
-    break; }
-  default:
-    break;
-  }
-  bool needs_back_conv = orig_type != my_gov;
-  // Try the template instance's explicit type specification.  This will be
-  // the winner if we already have the type.
-  if (p_ti->get_Type()) {
-    if (p_ti->get_Type()->get_type_refd_last() != my_gov) {
-      orig_type = p_ti->get_Type()->get_type_refd_last();
-      needs_back_conv = false;
-    }
-  }
-  if (!orig_type || !my_gov) FATAL_ERROR("TypeConv::gen_conv_code_redir()");
-  p_expr->preamble = mputprintf(p_expr->preamble,
-    "%s %s;\n"  // For back-conversion.
-    "%s %s;\n"  // To store the result of the operation, the "&()" part.
-    "%s %s;\n",
-    orig_type->get_genname_template(templ_body->get_my_scope()).c_str(),
-    tmp_id_str1, orig_type->get_genname_value(templ_body->get_my_scope())
-    .c_str(), tmp_id_str2, my_gov->get_genname_template(templ_body
-    ->get_my_scope()).c_str(), tmp_id_str3);
-  expr_tmp.expr = mputprintf(expr_tmp.expr, "%s = ",
-    !needs_back_conv ? tmp_id_str1 : tmp_id_str3);
-  p_ti->generate_code(&expr_tmp);
-  p_expr->preamble = Code::merge_free_expr(p_expr->preamble, &expr_tmp);
-  if (needs_back_conv) {
-    TypeConv *back_conv = new TypeConv(my_gov, orig_type, true);
-    templ_body->get_my_scope()->get_scope_mod()->add_type_conv(back_conv);
-    p_expr->preamble = mputprintf(p_expr->preamble,
-      "if (!%s(%s, %s)) TTCN_error(\"Values or templates of types `%s' and "
-      "`%s' are not compatible at run-time\");\n",
-      TypeConv::get_conv_func(my_gov, orig_type, templ_body->get_my_scope()
-      ->get_scope_mod()).c_str(), tmp_id_str1, tmp_id_str3, my_gov
-      ->get_typename().c_str(), orig_type->get_typename().c_str());
-  }
-  // The back-converted template is the first argument.
-  p_expr->expr = mputprintf(p_expr->expr, "%s, &(%s)",
-    tmp_id_str1, tmp_id_str2);
-  Code::init_expr(&expr_tmp);
-  p_ref->generate_code(&expr_tmp);
-  // Hand-merge 1.
-  if (expr_tmp.preamble)
-    p_expr->postamble = mputstr(p_expr->postamble, expr_tmp.preamble);
-  // Finally, convert back to the original value.  TODO: Simplify.  Don't
-  // let the conversion die due to an unsuccessful receive operation.
-  p_expr->postamble = mputprintf(p_expr->postamble,
-    "if (%s.is_bound() && !%s(%s, %s)) TTCN_error(\"Values or templates "
-    "of types `%s' and `%s' are not compatible at run-time\");\n",
-    tmp_id_str2, TypeConv::get_conv_func(orig_type, my_gov, templ_body
-    ->get_my_scope()->get_scope_mod()).c_str(), expr_tmp.expr, tmp_id_str2,
-    orig_type->get_typename().c_str(), my_gov->get_typename().c_str());
-  // Hand-merge 2.
-  if (expr_tmp.postamble)
-    p_expr->postamble = mputstr(p_expr->postamble, expr_tmp.postamble);
-  Code::free_expr(&expr_tmp);
-}
-
 bool TypeConv::needs_conv_refd(GovernedSimple *p_val_or_temp)
 {
   if (!use_runtime_2) FATAL_ERROR("TypeConv::needs_conv_refd()");
@@ -323,12 +202,12 @@ void TypeConv::gen_conv_func(char **p_prototypes, char **p_bodies,
   string to_name = m_is_temp ? m_to->get_genname_template(p_mod)
     : m_to->get_genname_value(p_mod);
   *p_prototypes = mputprintf(*p_prototypes,
-    "static boolean %s(%s& p_to_v, const %s& p_from_v);\n",
-    get_conv_func(m_from, m_to, p_mod).c_str(), to_name.c_str(),
+    "%sboolean %s(%s& p_to_v, const %s& p_from_v);\n",
+    split_to_slices ? "" : "static ", get_conv_func(m_from, m_to, p_mod).c_str(), to_name.c_str(),
     from_name.c_str());
   *p_bodies = mputprintf(*p_bodies,
-    "static boolean %s(%s& p_to_v, const %s& p_from_v)\n{\n",
-    get_conv_func(m_from, m_to, p_mod).c_str(), to_name.c_str(),
+    "%sboolean %s(%s& p_to_v, const %s& p_from_v)\n{\n",
+    split_to_slices ? "" : "static ", get_conv_func(m_from, m_to, p_mod).c_str(), to_name.c_str(),
     from_name.c_str());
   switch (m_to->get_typetype()) {
   case Type::T_SEQ_A:
@@ -603,7 +482,9 @@ void TypeConv::gen_conv_func_array_record_of(char **p_bodies, Module *p_mod)
   else if (m_to_tt == Type::T_SEQOF)
     *p_bodies = mputprintf(*p_bodies, "p_to_v.set_size(%lu);\n",
       (long unsigned)m_from->get_dimension()->get_size());
-  for (size_t i = 0; i < m_from->get_dimension()->get_size(); i++) {
+  size_t dim = m_from_tt == Type::T_ARRAY ? m_from->get_dimension()->get_size() :
+    m_to->get_dimension()->get_size();
+  for (size_t i = 0; i < dim; i++) {
     if (!p_mod->needs_type_conv(of_type_from, of_type_to)) {
       *p_bodies = mputprintf(*p_bodies,
         "if (p_from_v[%lu].is_bound()) p_to_v[%lu] = p_from_v[%lu];\n",
@@ -819,7 +700,7 @@ void TypeConv::gen_conv_func_choice_anytype(char **p_bodies, Module *p_mod)
             get_conv_func(cf_type_from, cf_type_to, p_mod).c_str(),
             tmp_id_str, anytype_prefix, cf_id_from.get_name().c_str(),
             tmp_id_str, tmp_id_str);
-        } else if (cf_type_from->is_compatible(cf_type_to, NULL)) {  // E.g. basic types.
+        } else if (cf_type_from->is_compatible(cf_type_to, NULL, NULL)) {  // E.g. basic types.
           // The same module + field name is required for anytype field
           // types.  Only for structured types.
           bool both_structured = cf_type_from->is_structured_type()
