@@ -925,7 +925,7 @@ namespace Ttcn {
       size_t nof_types = types_v.size();
       for (size_t i = 0; i < nof_types; i++) {
         // Don't allow type compatibility.
-        if (types_v[i]->is_compatible(p_type, NULL)) ret_val++;
+        if (types_v[i]->is_compatible(p_type, NULL, NULL)) ret_val++;
       }
       return ret_val;
     }
@@ -1816,6 +1816,8 @@ namespace Ttcn {
         pdef.msg_in.elements[i].name =
           pool.add(type->get_genname_value(my_scope));
         pdef.msg_in.elements[i].dispname = pool.add(type->get_typename());
+        pdef.msg_in.elements[i].name_w_no_prefix = pool.add(type->get_genname_value(
+          type->get_type_refd_last()->get_my_scope()));
       }
     } else {
       pdef.msg_in.nElements = 0;
@@ -1865,6 +1867,7 @@ namespace Ttcn {
           signature->is_nonblocking_signature();
         pdef.proc_in.elements[i].has_exceptions =
           signature->get_signature_exceptions() ? TRUE : FALSE;
+        pdef.proc_in.elements[i].has_return_value = FALSE;
       }
     } else {
       pdef.proc_in.nElements = 0;
@@ -1884,6 +1887,8 @@ namespace Ttcn {
           signature->is_nonblocking_signature();
         pdef.proc_out.elements[i].has_exceptions =
           signature->get_signature_exceptions() ? TRUE : FALSE;
+        pdef.proc_out.elements[i].has_return_value =
+          signature->get_signature_return_type() != NULL ? TRUE : FALSE;
       }
     } else {
       pdef.proc_out.nElements = 0;
