@@ -32,6 +32,7 @@
 #include "Error.hh"
 #include "Logger.hh"
 #include "Encdec.hh"
+#include "Addfunc.hh"
 
 #include "../common/dbgnew.hh"
 
@@ -559,9 +560,11 @@ void BITSTRING::log() const
 void BITSTRING::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_VALUE|Module_Param::BC_LIST, "bitstring value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Bitstring:
     switch (param.get_operation_type()) {
@@ -604,6 +607,7 @@ void BITSTRING::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* BITSTRING::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!is_bound()) {
@@ -614,6 +618,7 @@ Module_Param* BITSTRING::get_param(Module_Param_Name& /* param_name */) const
   memcpy(val_cpy, val_ptr->bits_ptr, n_bytes);
   return new Module_Param_Bitstring(val_ptr->n_bits, val_cpy);
 }
+#endif
 
 void BITSTRING::encode_text(Text_Buf& text_buf) const
 {
@@ -1099,6 +1104,11 @@ int BITSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
     for (; success == 1; success = reader.Read()) {
       type = reader.NodeType();
       if (XML_READER_TYPE_ELEMENT == type) {
+        // If our parent is optional and there is an unexpected tag then return and
+        // we stay unbound.
+        if ((flavor & XER_OPTIONAL) && !check_name((const char*)reader.LocalName(), p_td, exer)) {
+          return -1;
+        }
         verify_name(reader, p_td, exer);
         depth = reader.Depth();
         if (reader.IsEmptyElement()) {
@@ -1410,6 +1420,19 @@ void BITSTRING_template::clean_up()
     else TTCN_error("Internal error: Invalid reference counter in a bitstring "
       "pattern.");
     break;
+  case DECODE_MATCH:
+    if (dec_match->ref_count > 1) {
+      dec_match->ref_count--;
+    }
+    else if (dec_match->ref_count == 1) {
+      delete dec_match->instance;
+      delete dec_match;
+    }
+    else {
+      TTCN_error("Internal error: Invalid reference counter in a "
+        "decoded content match.");
+    }
+    break;
   default:
     break;
   }
@@ -1438,6 +1461,10 @@ void BITSTRING_template::copy_template(const BITSTRING_template& other_value)
     pattern_value = other_value.pattern_value;
     pattern_value->ref_count++;
     break;
+  case DECODE_MATCH:
+    dec_match = other_value.dec_match;
+    dec_match->ref_count++;
+    break;
   default:
     TTCN_error("Copying an uninitialized/unsupported bitstring template.");
   }
@@ -1693,6 +1720,15 @@ boolean BITSTRING_template::match(const BITSTRING& other_value,
     return template_selection == COMPLEMENTED_LIST;
   case STRING_PATTERN:
     return match_pattern(pattern_value, other_value.val_ptr);
+  case DECODE_MATCH: {
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL, TTCN_EncDec::EB_WARNING);
+    TTCN_EncDec::clear_error();
+    OCTETSTRING os(bit2oct(other_value));
+    TTCN_Buffer buff(os);
+    boolean ret_val = dec_match->instance->match(buff);
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL,TTCN_EncDec::EB_DEFAULT);
+    TTCN_EncDec::clear_error();
+    return ret_val; }
   default:
     TTCN_error("Matching an uninitialized/unsupported bitstring template.");
   }
@@ -1766,14 +1802,17 @@ int BITSTRING_template::lengthof() const
 }
 
 void BITSTRING_template::set_type(template_sel template_type,
-  unsigned int list_length)
+  unsigned int list_length /* = 0 */)
 {
-  if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST)
+  if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST &&
+      template_type != DECODE_MATCH)
     TTCN_error("Setting an invalid list type for a bitstring template.");
   clean_up();
   set_selection(template_type);
-  value_list.n_values = list_length;
-  value_list.list_value = new BITSTRING_template[list_length];
+  if (template_type != DECODE_MATCH) {
+    value_list.n_values = list_length;
+    value_list.list_value = new BITSTRING_template[list_length];
+  }
 }
 
 BITSTRING_template& BITSTRING_template::list_item(unsigned int list_index)
@@ -1786,6 +1825,35 @@ BITSTRING_template& BITSTRING_template::list_item(unsigned int list_index)
   return value_list.list_value[list_index];
 }
 
+void BITSTRING_template::set_decmatch(Dec_Match_Interface* new_instance)
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Setting the decoded content matching mechanism of a non-decmatch "
+      "bitstring template.");
+  }
+  dec_match = new decmatch_struct;
+  dec_match->ref_count = 1;
+  dec_match->instance = new_instance;
+}
+
+void* BITSTRING_template::get_decmatch_dec_res() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoding result of a non-decmatch bitstring "
+      "template.");
+  }
+  return dec_match->instance->get_dec_res();
+}
+
+const TTCN_Typedescriptor_t* BITSTRING_template::get_decmatch_type_descr() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoded type's descriptor in a non-decmatch "
+      "bitstring template.");
+  }
+  return dec_match->instance->get_type_descr();
+}
+
 static const char patterns[] = { '0', '1', '?', '*' };
 
 void BITSTRING_template::log() const
@@ -1814,6 +1882,10 @@ void BITSTRING_template::log() const
     }
     TTCN_Logger::log_event_str("'B");
     break;
+  case DECODE_MATCH:
+    TTCN_Logger::log_event_str("decmatch ");
+    dec_match->instance->log();
+    break;
   default:
     log_generic();
     break;
@@ -1840,9 +1912,11 @@ void BITSTRING_template::log_match(const BITSTRING& match_value,
 void BITSTRING_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE|Module_Param::BC_LIST, "bitstring template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -1892,6 +1966,7 @@ void BITSTRING_template::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* BITSTRING_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -1928,6 +2003,9 @@ Module_Param* BITSTRING_template::get_param(Module_Param_Name& param_name) const
     memcpy(val_cpy, pattern_value->elements_ptr, pattern_value->n_elements);
     mp = new Module_Param_Bitstring_Template(pattern_value->n_elements, val_cpy);
     break; }
+  case DECODE_MATCH:
+    mp->error("Referencing a decoded content matching template is not supported.");
+    break;
   default:
     break;
   }
@@ -1937,6 +2015,7 @@ Module_Param* BITSTRING_template::get_param(Module_Param_Name& param_name) const
   mp->set_length_restriction(get_length_range());
   return mp;
 }
+#endif
 
 void BITSTRING_template::encode_text(Text_Buf& text_buf) const
 {
