@@ -181,14 +181,17 @@ void BOOLEAN::decode_text(Text_Buf& text_buf)
 void BOOLEAN::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_VALUE, "boolean value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   if (mp->get_type()!=Module_Param::MP_Boolean) param.type_error("boolean value");
   bound_flag = TRUE;
   boolean_value = mp->get_boolean();
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* BOOLEAN::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!is_bound()) {
@@ -196,6 +199,7 @@ Module_Param* BOOLEAN::get_param(Module_Param_Name& /* param_name */) const
   }
   return new Module_Param_Boolean(boolean_value);
 }
+#endif
 
 void BOOLEAN::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
                      TTCN_EncDec::coding_t p_coding, ...) const
@@ -659,6 +663,11 @@ int BOOLEAN::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
       type = reader.NodeType();
       if (!notag && depth == -1) {
         if (XML_READER_TYPE_ELEMENT == type) {
+          // If our parent is optional and there is an unexpected tag then return and
+          // we stay unbound.
+          if ((flavor & XER_OPTIONAL) && !check_name((const char*)reader.LocalName(), p_td, exer)) {
+            return -1;
+          }
           verify_name(reader, p_td, exer);
           depth = reader.Depth();
 
@@ -1037,9 +1046,11 @@ void BOOLEAN_template::log_match(const BOOLEAN& match_value,
 void BOOLEAN_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE, "boolean template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -1069,6 +1080,7 @@ void BOOLEAN_template::set_param(Module_Param& param) {
   is_ifpresent = param.get_ifpresent() || mp->get_ifpresent();
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* BOOLEAN_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -1108,6 +1120,7 @@ Module_Param* BOOLEAN_template::get_param(Module_Param_Name& param_name) const
   }
   return mp;
 }
+#endif
 
 void BOOLEAN_template::encode_text(Text_Buf& text_buf) const
 {
