@@ -208,17 +208,22 @@ CHARSTRING& CHARSTRING::operator=(const UNIVERSAL_CHARSTRING& other_value)
 {
   other_value.must_bound("Assignment of an unbound universal charstring to "
                          "a charstring.");
-  clean_up();
-  int n_chars = other_value.val_ptr->n_uchars;
-  init_struct(n_chars);
-  for (int i = 0; i < n_chars; ++i) {
-    const universal_char& uc = other_value.val_ptr->uchars_ptr[i];
-    if (uc.uc_group != 0 || uc.uc_plane != 0 || uc.uc_row != 0) {
-      TTCN_error("Multiple-byte characters cannot be assigned to a charstring, "
-        "invalid character char(%u, %u, %u, %u) at index %d.", 
-        uc.uc_group, uc.uc_plane, uc.uc_row, uc.uc_cell, i);
+  if (other_value.charstring) {
+    *this = other_value.cstr;
+  }
+  else {
+    clean_up();
+    int n_chars = other_value.val_ptr->n_uchars;
+    init_struct(n_chars);
+    for (int i = 0; i < n_chars; ++i) {
+      const universal_char& uc = other_value.val_ptr->uchars_ptr[i];
+      if (uc.uc_group != 0 || uc.uc_plane != 0 || uc.uc_row != 0) {
+        TTCN_error("Multiple-byte characters cannot be assigned to a charstring, "
+          "invalid character char(%u, %u, %u, %u) at index %d.", 
+          uc.uc_group, uc.uc_plane, uc.uc_row, uc.uc_cell, i);
+      }
+      val_ptr->chars_ptr[i] = other_value.val_ptr->uchars_ptr[i].uc_cell;
     }
-    val_ptr->chars_ptr[i] = other_value.val_ptr->uchars_ptr[i].uc_cell;
   }
   return *this;
 }
@@ -644,13 +649,16 @@ void CHARSTRING::log() const
   }
 }
 
-boolean CHARSTRING::set_param_internal(Module_Param& param, boolean allow_pattern) {
+boolean CHARSTRING::set_param_internal(Module_Param& param, boolean allow_pattern,
+                                       boolean* is_nocase_pattern) {
   boolean is_pattern = FALSE;
   param.basic_check(Module_Param::BC_VALUE|Module_Param::BC_LIST, "charstring value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Universal_Charstring:
   case Module_Param::MP_Charstring:
@@ -697,7 +705,8 @@ boolean CHARSTRING::set_param_internal(Module_Param& param, boolean allow_patter
     if (mp->get_expr_type() == Module_Param::EXPR_CONCATENATE) {
       // only allow string patterns for the first operand
       CHARSTRING operand1, operand2;
-      is_pattern = operand1.set_param_internal(*mp->get_operand1(), allow_pattern);
+      is_pattern = operand1.set_param_internal(*mp->get_operand1(), allow_pattern,
+        is_nocase_pattern);
       operand2.set_param(*mp->get_operand2());
       if (param.get_operation_type() == Module_Param::OT_CONCAT) {
         *this = *this + operand1 + operand2;
@@ -714,6 +723,9 @@ boolean CHARSTRING::set_param_internal(Module_Param& param, boolean allow_patter
     if (allow_pattern) {
       *this = CHARSTRING(mp->get_pattern());
       is_pattern = TRUE;
+      if (is_nocase_pattern != NULL) {
+        *is_nocase_pattern = mp->get_nocase();
+      }
       break;
     }
     // else fall through
@@ -728,6 +740,7 @@ void CHARSTRING::set_param(Module_Param& param) {
   set_param_internal(param, FALSE);
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* CHARSTRING::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!is_bound()) {
@@ -735,6 +748,7 @@ Module_Param* CHARSTRING::get_param(Module_Param_Name& /* param_name */) const
   }
   return new Module_Param_Charstring(val_ptr->n_chars, mcopystr(val_ptr->chars_ptr));
 }
+#endif
 
 void CHARSTRING::encode_text(Text_Buf& text_buf) const
 {
@@ -1358,6 +1372,11 @@ int CHARSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
     for (; success == 1; success = reader.Read()) {
       int type = reader.NodeType();
       if (XML_READER_TYPE_ELEMENT == type) {
+        // If our parent is optional and there is an unexpected tag then return and
+        // we stay unbound.
+        if ((flavor & XER_OPTIONAL) && !check_name((const char*)reader.LocalName(), p_td, exer)) {
+          return -1;
+        }
         verify_name(reader, p_td, exer);
         if (reader.IsEmptyElement()) { // has no text, needs special processing
           if (exer && p_td.dfeValue != 0) {
@@ -2007,6 +2026,19 @@ void CHARSTRING_template::clean_up()
   case STRING_PATTERN:
     if (pattern_value.regexp_init) regfree(&pattern_value.posix_regexp);
     break;
+  case DECODE_MATCH:
+    if (dec_match->ref_count > 1) {
+      dec_match->ref_count--;
+    }
+    else if (dec_match->ref_count == 1) {
+      delete dec_match->instance;
+      delete dec_match;
+    }
+    else {
+      TTCN_error("Internal error: Invalid reference counter in a "
+        "decoded content match.");
+    }
+    break;
   default:
     break;
   }
@@ -2018,6 +2050,7 @@ void CHARSTRING_template::copy_template(const CHARSTRING_template& other_value)
   switch (other_value.template_selection) {
   case STRING_PATTERN:
     pattern_value.regexp_init = FALSE;
+    pattern_value.nocase = other_value.pattern_value.nocase;
     /* no break */
   case SPECIFIC_VALUE:
     single_value = other_value.single_value;
@@ -2041,6 +2074,10 @@ void CHARSTRING_template::copy_template(const CHARSTRING_template& other_value)
       "not set when copying a charstring value range template.");
     value_range = other_value.value_range;
     break;
+  case DECODE_MATCH:
+    dec_match = other_value.dec_match;
+    dec_match->ref_count++;
+    break;
   default:
     TTCN_error("Copying an uninitialized/unsupported charstring template.");
   }
@@ -2084,13 +2121,15 @@ CHARSTRING_template::CHARSTRING_template(const OPTIONAL<CHARSTRING>& other_value
 }
 
 CHARSTRING_template::CHARSTRING_template(template_sel p_sel,
-                                         const CHARSTRING& p_str)
+                                         const CHARSTRING& p_str,
+                                         boolean p_nocase /* = FALSE */)
   : Restricted_Length_Template(STRING_PATTERN), single_value(p_str)
 {
   if(p_sel!=STRING_PATTERN)
     TTCN_error("Internal error: Initializing a charstring pattern template "
                "with invalid selection.");
   pattern_value.regexp_init=FALSE;
+  pattern_value.nocase = p_nocase;
 }
 
 CHARSTRING_template::CHARSTRING_template(const CHARSTRING_template& other_value)
@@ -2216,8 +2255,11 @@ boolean CHARSTRING_template::match(const CHARSTRING& other_value,
       TTCN_Logger::log_event(" is: \"%s\"", posix_str);
       //TTCN_Logger::end_event();
       */
-      int ret_val=regcomp(&pattern_value.posix_regexp, posix_str,
-                      REG_EXTENDED|REG_NOSUB);
+      int flags = REG_EXTENDED|REG_NOSUB;
+      if (pattern_value.nocase) {
+        flags |= REG_ICASE;
+      }
+      int ret_val=regcomp(&pattern_value.posix_regexp, posix_str, flags);
       Free(posix_str);
       if(ret_val!=0) {
         /* regexp error */
@@ -2268,6 +2310,14 @@ boolean CHARSTRING_template::match(const CHARSTRING& other_value,
     }
     return TRUE;
     break; }
+  case DECODE_MATCH: {
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL, TTCN_EncDec::EB_WARNING);
+    TTCN_EncDec::clear_error();
+    TTCN_Buffer buff(other_value);
+    boolean ret_val = dec_match->instance->match(buff);
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL,TTCN_EncDec::EB_DEFAULT);
+    TTCN_EncDec::clear_error();
+    return ret_val; }
   default:
     TTCN_error("Matching with an uninitialized/unsupported charstring "
                "template.");
@@ -2352,6 +2402,9 @@ void CHARSTRING_template::set_type(template_sel template_type,
     value_range.min_is_set = FALSE;
     value_range.max_is_set = FALSE;
     break;
+  case DECODE_MATCH:
+    set_selection(DECODE_MATCH);
+    break;
   default:
     TTCN_error("Setting an invalid type for a charstring template.");
   }
@@ -2403,9 +2456,46 @@ void CHARSTRING_template::set_max(const CHARSTRING& max_value)
       value_range.min_value);
 }
 
-void CHARSTRING_template::log_pattern(int n_chars, const char *chars_ptr)
+void CHARSTRING_template::set_decmatch(Dec_Match_Interface* new_instance)
 {
-  TTCN_Logger::log_event_str("pattern \"");
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Setting the decoded content matching mechanism of a non-decmatch "
+      "charstring template.");
+  }
+  dec_match = new unichar_decmatch_struct;
+  dec_match->ref_count = 1;
+  dec_match->instance = new_instance;
+  // the character coding is only used if this template is copied to a
+  // universal charstring template
+  dec_match->coding = CharCoding::UTF_8;
+}
+
+void* CHARSTRING_template::get_decmatch_dec_res() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoding result of a non-decmatch charstring "
+      "template.");
+  }
+  return dec_match->instance->get_dec_res();
+}
+
+const TTCN_Typedescriptor_t* CHARSTRING_template::get_decmatch_type_descr() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoded type's descriptor in a non-decmatch "
+      "charstring template.");
+  }
+  return dec_match->instance->get_type_descr();
+}
+
+void CHARSTRING_template::log_pattern(int n_chars, const char *chars_ptr,
+                                      boolean nocase)
+{
+  TTCN_Logger::log_event_str("pattern ");
+  if (nocase) {
+    TTCN_Logger::log_event_str("@nocase ");
+  }
+  TTCN_Logger::log_event_str("\"");
   enum { INITIAL, BACKSLASH, BACKSLASH_Q, QUADRUPLE, HASHMARK, REPETITIONS }
     state = INITIAL;
   for (int i = 0; i < n_chars; i++) {
@@ -2530,7 +2620,8 @@ void CHARSTRING_template::log() const
 {
   switch (template_selection) {
   case STRING_PATTERN:
-    log_pattern(single_value.lengthof(), (const char*)single_value);
+    log_pattern(single_value.lengthof(), (const char*)single_value,
+      pattern_value.nocase);
     break;
   case SPECIFIC_VALUE:
     single_value.log();
@@ -2567,6 +2658,10 @@ void CHARSTRING_template::log() const
     } else TTCN_Logger::log_event_str("<unknown upper bound>");
     TTCN_Logger::log_char(')');
     break;
+  case DECODE_MATCH:
+    TTCN_Logger::log_event_str("decmatch ");
+    dec_match->instance->log();
+    break;
   default:
     log_generic();
   }
@@ -2592,9 +2687,11 @@ void CHARSTRING_template::log_match(const CHARSTRING& match_value,
 void CHARSTRING_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE|Module_Param::BC_LIST, "charstring template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -2634,19 +2731,23 @@ void CHARSTRING_template::set_param(Module_Param& param) {
     clean_up();
     single_value = CHARSTRING(mp->get_pattern());
     pattern_value.regexp_init = FALSE;
+    pattern_value.nocase = mp->get_nocase();
     set_selection(STRING_PATTERN);
     break;
   case Module_Param::MP_Expression:
     if (mp->get_expr_type() == Module_Param::EXPR_CONCATENATE) {
       // only allow string patterns for the first operand
       CHARSTRING operand1, operand2, result;
-      boolean is_pattern = operand1.set_param_internal(*mp->get_operand1(), TRUE);
+      boolean nocase;
+      boolean is_pattern = operand1.set_param_internal(*mp->get_operand1(),
+        TRUE, &nocase);
       operand2.set_param(*mp->get_operand2());
       result = operand1 + operand2;
       if (is_pattern) {
         clean_up();
         single_value = result;
         pattern_value.regexp_init = FALSE;
+        pattern_value.nocase = nocase;
         set_selection(STRING_PATTERN);
       }
       else {
@@ -2669,6 +2770,7 @@ void CHARSTRING_template::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* CHARSTRING_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -2706,7 +2808,10 @@ Module_Param* CHARSTRING_template::get_param(Module_Param_Name& param_name) cons
     mp = new Module_Param_StringRange(lower_bound, upper_bound);
     break; }
   case STRING_PATTERN:
-    mp = new Module_Param_Pattern(mcopystr(single_value));
+    mp = new Module_Param_Pattern(mcopystr(single_value), pattern_value.nocase);
+    break;
+  case DECODE_MATCH:
+    mp->error("Referencing a decoded content matching template is not supported.");
     break;
   default:
     break;
@@ -2717,6 +2822,7 @@ Module_Param* CHARSTRING_template::get_param(Module_Param_Name& param_name) cons
   mp->set_length_restriction(get_length_range());
   return mp;
 }
+#endif
 
 void CHARSTRING_template::encode_text(Text_Buf& text_buf) const
 {
@@ -2726,8 +2832,10 @@ void CHARSTRING_template::encode_text(Text_Buf& text_buf) const
   case ANY_VALUE:
   case ANY_OR_OMIT:
     break;
-  case SPECIFIC_VALUE:
   case STRING_PATTERN:
+    text_buf.push_int(pattern_value.nocase);
+    // no break;
+  case SPECIFIC_VALUE:
     single_value.encode_text(text_buf);
     break;
   case VALUE_LIST:
@@ -2761,6 +2869,7 @@ void CHARSTRING_template::decode_text(Text_Buf& text_buf)
     break;
   case STRING_PATTERN:
     pattern_value.regexp_init=FALSE;
+    pattern_value.nocase = text_buf.pull_int().get_val();
     /* no break */
   case SPECIFIC_VALUE:
     single_value.decode_text(text_buf);
