@@ -262,9 +262,11 @@ void FLOAT::log() const
 void FLOAT::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_VALUE, "float value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Float: {
     clean_up();
@@ -316,6 +318,7 @@ void FLOAT::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* FLOAT::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!bound_flag) {
@@ -323,6 +326,7 @@ Module_Param* FLOAT::get_param(Module_Param_Name& /* param_name */) const
   }
   return new Module_Param_Float(float_value);
 }
+#endif
 
 void FLOAT::encode_text(Text_Buf& text_buf) const
 {
@@ -839,6 +843,10 @@ int FLOAT::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
   return decode_length + prepaddlength;
 }
 
+const char* XER_POS_INF_STR = "INF";
+const char* XER_NEG_INF_STR = "-INF";
+const char* XER_NAN_STR = "NaN";
+
 int FLOAT::XER_encode(const XERdescriptor_t& p_td,
   TTCN_Buffer& p_buf, unsigned int flavor, int indent, embed_values_enc_struct_t*) const
 {
@@ -855,11 +863,37 @@ int FLOAT::XER_encode(const XERdescriptor_t& p_td,
 
   if (exer && (p_td.xer_bits & XER_DECIMAL)) {
     char buf[312];
-    int n = snprintf(buf, sizeof(buf), "%f", (double)float_value);
+    int n = 0;
+    if (isnan((double)float_value)) {
+      n = snprintf(buf, sizeof(buf), "%s", XER_NAN_STR);
+    } else if ((double)float_value == (double)INFINITY) {
+      n = snprintf(buf, sizeof(buf), "%s", XER_POS_INF_STR);
+    } else if ((double)float_value == -(double)INFINITY) {
+      n = snprintf(buf, sizeof(buf), "%s", XER_NEG_INF_STR);
+    } else {
+      n = snprintf(buf, sizeof(buf), "%f", (double)float_value);
+      if (p_td.fractionDigits != -1) {
+        char *p = strchr(buf, '.');
+        if (p != NULL) {
+          int offset = p_td.fractionDigits == 0 ? 0 : p_td.fractionDigits + 1;
+          p[offset] = 0;
+          n = strlen(buf);
+        }
+      }
+    }
     p_buf.put_s((size_t)n, (const unsigned char*)buf);
   }
   else {
-    CHARSTRING value = float2str(float_value);
+    CHARSTRING value;
+    if (isnan((double)float_value)) {
+      value = XER_NAN_STR;
+    } else if ((double)float_value == (double)INFINITY) {
+      value = XER_POS_INF_STR;
+    } else if ((double)float_value == -(double)INFINITY) {
+      value = XER_NEG_INF_STR;
+    } else {
+      value = float2str(float_value);
+    }
     p_buf.put_string(value);
   }
 
@@ -938,18 +972,48 @@ int FLOAT::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
     verify_name(reader, p_td, exer);
 tagless:
     const char * value = (const char *)reader.Value();
-
-    if (value && is_float(value)) {
-      bound_flag = true;
-      sscanf(value, "%lf", &float_value);
+    if (value) {
+      if (is_float(value)) {
+        if (exer && (p_td.xer_bits & XER_DECIMAL) && p_td.fractionDigits != -1) {
+          char *p = strchr((char*)value, '.');
+          if (p != NULL) {
+            unsigned int fraction_digits_pos = (int)(p - value) + 1 + p_td.fractionDigits;
+            if (fraction_digits_pos < strlen(value)) {
+              TTCN_EncDec_ErrorContext::error(TTCN_EncDec::ET_FLOAT_TR,
+                "The float value (%s) contains too many fractionDigits. Expected %i or less.",
+                value,
+                p_td.fractionDigits);
+            }
+          }
+        }
+        bound_flag = true;
+        sscanf(value, "%lf", &float_value);
+      } else if (strcmp(XER_NAN_STR, value) == 0 ) {
+        bound_flag = true;
+#ifdef NAN
+        float_value = NAN;
+#else
+        float_value = INFINITY + (-INFINITY);
+#endif
+      } else if (strcmp(XER_POS_INF_STR, value) == 0) {
+        bound_flag = true;
+        float_value = (double)INFINITY;
+      } else if (strcmp(XER_NEG_INF_STR, value) == 0) {
+        bound_flag = true;
+        float_value = -(double)INFINITY;
+      }
     }
-
     // Let the caller do reader.AdvanceAttribute();
   }
   else {
     for (; success == 1; success = reader.Read()) {
       int type = reader.NodeType();
       if (XML_READER_TYPE_ELEMENT == type) {
+        // If our parent is optional and there is an unexpected tag then return and
+        // we stay unbound.
+        if ((flavor & XER_OPTIONAL) && !check_name((const char*)reader.LocalName(), p_td, exer)) {
+          return -1;
+        }
         verify_name(reader, p_td, exer);
         if (reader.IsEmptyElement()) {
           if (exer && p_td.dfeValue != 0) {
@@ -962,9 +1026,36 @@ tagless:
       }
       else if (XML_READER_TYPE_TEXT == type && depth != -1) {
         const char * value = (const char*)reader.Value();
-        if (value && is_float(value)) {
-          bound_flag = true;
-          sscanf(value, "%lf", &float_value);
+        if (value) {
+          if (is_float(value)) {
+            if (exer && (p_td.xer_bits & XER_DECIMAL) && p_td.fractionDigits != -1) {
+              char *p = strchr((char*)value, '.');
+              if (p != NULL) {
+                unsigned int fraction_digits_pos = (int)(p - value) + 1 + p_td.fractionDigits;
+                if (fraction_digits_pos < strlen(value)) {
+                  TTCN_EncDec_ErrorContext::error(TTCN_EncDec::ET_FLOAT_TR,
+                    "The float value (%s) contains too many fractionDigits. Expected %i or less.",
+                    value,
+                    p_td.fractionDigits);
+                }
+              }
+            }
+            bound_flag = true;
+            sscanf(value, "%lf", &float_value);
+          } else if (strcmp("NaN", value) == 0 ) {
+            bound_flag = true;
+#ifdef NAN
+            float_value = NAN;
+#else
+            float_value = INFINITY + (-INFINITY);
+#endif
+          } else if (strcmp("INF", value) == 0) {
+            bound_flag = true;
+            float_value = (double)INFINITY;
+          } else if (strcmp("-INF", value) == 0) {
+            bound_flag = true;
+            float_value = -(double)INFINITY;
+          }
         }
       }
       else if (XML_READER_TYPE_END_ELEMENT == type) {
@@ -1431,9 +1522,11 @@ void FLOAT_template::log_match(const FLOAT& match_value,
 void FLOAT_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE, "float template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -1507,6 +1600,7 @@ void FLOAT_template::set_param(Module_Param& param) {
   is_ifpresent = param.get_ifpresent() || mp->get_ifpresent();
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* FLOAT_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -1551,6 +1645,7 @@ Module_Param* FLOAT_template::get_param(Module_Param_Name& param_name) const
   }
   return mp;
 }
+#endif
 
 void FLOAT_template::encode_text(Text_Buf& text_buf) const
 {
