@@ -600,9 +600,11 @@ void HEXSTRING::decode_text(Text_Buf& text_buf)
 void HEXSTRING::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_VALUE|Module_Param::BC_LIST, "hexstring value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Hexstring:
     switch (param.get_operation_type()) {
@@ -646,6 +648,7 @@ void HEXSTRING::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* HEXSTRING::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!is_bound()) {
@@ -656,6 +659,7 @@ Module_Param* HEXSTRING::get_param(Module_Param_Name& /* param_name */) const
   memcpy(val_cpy, val_ptr->nibbles_ptr, n_bytes);
   return new Module_Param_Hexstring(val_ptr->n_nibbles, val_cpy);
 }
+#endif
 
 void HEXSTRING::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
   TTCN_EncDec::coding_t p_coding, ...) const
@@ -933,6 +937,11 @@ int HEXSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
    if (own_tag) for (; success == 1; success = reader.Read()) {
     type = reader.NodeType();
     if (XML_READER_TYPE_ELEMENT == type) {
+      // If our parent is optional and there is an unexpected tag then return and
+      // we stay unbound.
+      if ((flavor & XER_OPTIONAL) && !check_name((const char*)reader.LocalName(), p_td, exer)) {
+        return -1;
+      }
       verify_name(reader, p_td, exer);
       depth = reader.Depth();
       if (reader.IsEmptyElement()) {
@@ -1304,6 +1313,19 @@ void HEXSTRING_template::clean_up()
       TTCN_error("Internal error: Invalid reference counter in a hexstring "
         "pattern.");
     break;
+  case DECODE_MATCH:
+    if (dec_match->ref_count > 1) {
+      dec_match->ref_count--;
+    }
+    else if (dec_match->ref_count == 1) {
+      delete dec_match->instance;
+      delete dec_match;
+    }
+    else {
+      TTCN_error("Internal error: Invalid reference counter in a "
+        "decoded content match.");
+    }
+    break;
   default:
     break;
   }
@@ -1332,6 +1354,10 @@ void HEXSTRING_template::copy_template(const HEXSTRING_template& other_value)
     pattern_value = other_value.pattern_value;
     pattern_value->ref_count++;
     break;
+  case DECODE_MATCH:
+    dec_match = other_value.dec_match;
+    dec_match->ref_count++;
+    break;
   default:
     TTCN_error("Copying an uninitialized/unsupported hexstring template.");
   }
@@ -1582,6 +1608,15 @@ boolean HEXSTRING_template::match(const HEXSTRING& other_value,
     return template_selection == COMPLEMENTED_LIST;
   case STRING_PATTERN:
     return match_pattern(pattern_value, other_value.val_ptr);
+  case DECODE_MATCH: {
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL, TTCN_EncDec::EB_WARNING);
+    TTCN_EncDec::clear_error();
+    OCTETSTRING os(hex2oct(other_value));
+    TTCN_Buffer buff(os);
+    boolean ret_val = dec_match->instance->match(buff);
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL,TTCN_EncDec::EB_DEFAULT);
+    TTCN_EncDec::clear_error();
+    return ret_val; }
   default:
     TTCN_error("Matching an uninitialized/unsupported hexstring template.");
   }
@@ -1655,12 +1690,15 @@ int HEXSTRING_template::lengthof() const
 void HEXSTRING_template::set_type(template_sel template_type,
   unsigned int list_length)
 {
-  if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST) TTCN_error(
+  if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST &&
+      template_type != DECODE_MATCH) TTCN_error(
     "Setting an invalid list type for a hexstring template.");
   clean_up();
   set_selection(template_type);
-  value_list.n_values = list_length;
-  value_list.list_value = new HEXSTRING_template[list_length];
+  if (template_type != DECODE_MATCH) {
+    value_list.n_values = list_length;
+    value_list.list_value = new HEXSTRING_template[list_length];
+  }
 }
 
 HEXSTRING_template& HEXSTRING_template::list_item(unsigned int list_index)
@@ -1673,6 +1711,35 @@ HEXSTRING_template& HEXSTRING_template::list_item(unsigned int list_index)
   return value_list.list_value[list_index];
 }
 
+void HEXSTRING_template::set_decmatch(Dec_Match_Interface* new_instance)
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Setting the decoded content matching mechanism of a non-decmatch "
+      "hexstring template.");
+  }
+  dec_match = new decmatch_struct;
+  dec_match->ref_count = 1;
+  dec_match->instance = new_instance;
+}
+
+void* HEXSTRING_template::get_decmatch_dec_res() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoding result of a non-decmatch hexstring "
+      "template.");
+  }
+  return dec_match->instance->get_dec_res();
+}
+
+const TTCN_Typedescriptor_t* HEXSTRING_template::get_decmatch_type_descr() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoded type's descriptor in a non-decmatch "
+      "hexstring template.");
+  }
+  return dec_match->instance->get_type_descr();
+}
+
 void HEXSTRING_template::log() const
 {
   switch (template_selection) {
@@ -1705,6 +1772,10 @@ void HEXSTRING_template::log() const
     }
     TTCN_Logger::log_event_str("'H");
     break;
+  case DECODE_MATCH:
+    TTCN_Logger::log_event_str("decmatch ");
+    dec_match->instance->log();
+    break;
   default:
     log_generic();
     break;
@@ -1733,9 +1804,11 @@ void HEXSTRING_template::log_match(const HEXSTRING& match_value,
 void HEXSTRING_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE|Module_Param::BC_LIST, "hexstring template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -1785,6 +1858,7 @@ void HEXSTRING_template::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* HEXSTRING_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -1821,6 +1895,9 @@ Module_Param* HEXSTRING_template::get_param(Module_Param_Name& param_name) const
     memcpy(val_cpy, pattern_value->elements_ptr, pattern_value->n_elements);
     mp = new Module_Param_Hexstring_Template(pattern_value->n_elements, val_cpy);
     break; }
+  case DECODE_MATCH:
+    mp->error("Referencing a decoded content matching template is not supported.");
+    break;
   default:
     break;
   }
@@ -1830,6 +1907,7 @@ Module_Param* HEXSTRING_template::get_param(Module_Param_Name& param_name) const
   mp->set_length_restriction(get_length_range());
   return mp;
 }
+#endif
 
 void HEXSTRING_template::encode_text(Text_Buf& text_buf) const
 {
