@@ -598,11 +598,11 @@ long long int INTEGER::get_long_long_val() const
   // It feels so bad accessing a BIGNUM directly, but faster than string
   // conversion...
   // I know, I had to fix this... Bence
-  if (BN_num_bytes(val.openssl) <= sizeof(BN_ULONG)) {
+  if ((size_t)BN_num_bytes(val.openssl) <= sizeof(BN_ULONG)) {
     return !is_negative ? BN_get_word(val.openssl) : -BN_get_word(val.openssl);
   }
   
-  unsigned num_bytes = BN_num_bytes(val.openssl);
+  int num_bytes = BN_num_bytes(val.openssl);
   unsigned char* tmp = (unsigned char*)Malloc(num_bytes * sizeof(unsigned char));
   BN_bn2bin(val.openssl, tmp);
   ret_val = tmp[0] & 0xff;
@@ -657,9 +657,11 @@ void INTEGER::set_param(Module_Param& param)
 {
   param.basic_check(Module_Param::BC_VALUE, "integer value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Integer: {
     clean_up();
@@ -717,6 +719,7 @@ void INTEGER::set_param(Module_Param& param)
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* INTEGER::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!bound_flag) {
@@ -727,6 +730,7 @@ Module_Param* INTEGER::get_param(Module_Param_Name& /* param_name */) const
   }
   return new Module_Param_Integer(new int_val_t(BN_dup(val.openssl)));
 }
+#endif
 
 void INTEGER::encode_text(Text_Buf& text_buf) const
 {
@@ -1308,7 +1312,7 @@ int INTEGER::RAW_encode_openssl(const TTCN_Typedescriptor_t& p_td,
   // Conversion to 2's complement.
   if (twos_compl) {
     BN_set_negative(D, 0);
-    unsigned num_bytes = BN_num_bytes(D);
+    int num_bytes = BN_num_bytes(D);
     unsigned char* tmp = (unsigned char*)Malloc(num_bytes * sizeof(unsigned char));
     BN_bn2bin(D, tmp);
     for (int a = 0; a < num_bytes; a++) tmp[a] = ~tmp[a];
@@ -1643,6 +1647,11 @@ tagless:
     for (; success == 1; success = reader.Read()) {
       type = reader.NodeType();
       if (XML_READER_TYPE_ELEMENT == type) {
+        // If our parent is optional and there is an unexpected tag then return and
+        // we stay unbound.
+        if ((flavor & XER_OPTIONAL) && !check_name((const char*)reader.LocalName(), p_td, exer)) {
+          return -1;
+        }
         verify_name(reader, p_td, exer);
         if (reader.IsEmptyElement()) {
           if (exer && p_td.dfeValue != 0) {
@@ -2343,9 +2352,11 @@ void INTEGER_template::log_match(const INTEGER& match_value,
 void INTEGER_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE, "integer template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -2429,6 +2440,7 @@ void INTEGER_template::set_param(Module_Param& param) {
   is_ifpresent = param.get_ifpresent() || mp->get_ifpresent();
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* INTEGER_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -2494,6 +2506,7 @@ Module_Param* INTEGER_template::get_param(Module_Param_Name& param_name) const
   }
   return mp;
 }
+#endif
 
 void INTEGER_template::encode_text(Text_Buf& text_buf) const
 {
