@@ -157,6 +157,12 @@ void Module_List::initialize_component(const char *module_name,
 
 void Module_List::set_param(Module_Param& param)
 {
+#ifndef TITAN_RUNTIME_2
+    if (param.get_id()->get_nof_names() > (size_t)2) {
+      param.error("Module parameter cannot be set. Field names and array "
+        "indexes are not supported in the Load Test Runtime.");
+    }
+#endif
   // The first segment in the parameter name can either be the module name,
   // or the module parameter name - both must be checked
   const char* const first_name = param.get_id()->get_current_name();
@@ -175,6 +181,25 @@ void Module_List::set_param(Module_Param& param)
   // If not found, check if the first name segment was the module parameter name
   // (even if it matched a module name)
   if (!param_found) {
+#ifndef TITAN_RUNTIME_2
+    if (param.get_id()->get_nof_names() == (size_t)2) {
+      const char* note = "(Note: field names and array indexes are not "
+        "supported in the Load Test Runtime).";
+      if (module_ptr == NULL) {
+        param.error("Module parameter cannot be set, because module '%s' "
+          "does not exist. %s", first_name, note);
+      }
+      else if (module_ptr->set_param_func == NULL) {
+        param.error("Module parameter cannot be set, because module '%s' "
+          "does not have parameters. %s", first_name, note);
+      }
+      else {
+        param.error("Module parameter cannot be set, because no parameter with "
+          " name '%s' exists in module '%s'. %s", second_name, first_name, note);
+      }
+    }
+#endif
+    
     param.get_id()->next_name(-1); // set the position back to the first segment
     for (TTCN_Module *list_iter = list_head; list_iter != NULL;
       list_iter = list_iter->list_next) {
@@ -203,7 +228,9 @@ void Module_List::set_param(Module_Param& param)
   }
 }
 
-Module_Param* Module_List::get_param(Module_Param_Name& param_name)
+#ifdef TITAN_RUNTIME_2
+Module_Param* Module_List::get_param(Module_Param_Name& param_name,
+                                     const Module_Param* caller)
 {
   // The first segment in the parameter name can either be the module name,
   // or the module parameter name - both must be checked
@@ -238,26 +265,27 @@ Module_Param* Module_List::get_param(Module_Param_Name& param_name)
   // Still not found -> error
   if (param == NULL) {
     if (module_ptr == NULL) {
-      TTCN_error("Referenced module parameter cannot be found. Module `%s' does not exist, "
+      caller->error("Referenced module parameter cannot be found. Module `%s' does not exist, "
         "and no parameter with name `%s' exists in any module.", 
         first_name, first_name);
     } else if (module_ptr->get_param_func == NULL) {
-      TTCN_error("Referenced module parameter cannot be found. Module `%s' does not have "
+      caller->error("Referenced module parameter cannot be found. Module `%s' does not have "
         "parameters, and no parameter with name `%s' exists in other modules.", 
         first_name, first_name);
     } else {
-      TTCN_error("Referenced module parameter cannot be found. No parameter with name `%s' "
+      caller->error("Referenced module parameter cannot be found. No parameter with name `%s' "
         "exists in module `%s', and no parameter with name `%s' exists in any module.",
         second_name, first_name, first_name);
     }
   }
   else if (param->get_type() == Module_Param::MP_Unbound) {
     delete param;
-    TTCN_error("Referenced module parameter '%s' is unbound.", param_name.get_str());
+    caller->error("Referenced module parameter '%s' is unbound.", param_name.get_str());
   }
 
   return param;
 }
+#endif
 
 void Module_List::log_param()
 {
