@@ -517,9 +517,11 @@ void OCTETSTRING::log() const
 void OCTETSTRING::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_VALUE|Module_Param::BC_LIST, "octetstring value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Octetstring:
     switch (param.get_operation_type()) {
@@ -561,6 +563,7 @@ void OCTETSTRING::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* OCTETSTRING::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!is_bound()) {
@@ -570,6 +573,7 @@ Module_Param* OCTETSTRING::get_param(Module_Param_Name& /* param_name */) const
   memcpy(val_cpy, val_ptr->octets_ptr, val_ptr->n_octets);
   return new Module_Param_Octetstring(val_ptr->n_octets, val_cpy);
 }
+#endif
 
 void OCTETSTRING::encode_text(Text_Buf& text_buf) const
 {
@@ -971,6 +975,11 @@ int OCTETSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
    if (own_tag) for (; success == 1; success = reader.Read()) {
     type = reader.NodeType();
     if (XML_READER_TYPE_ELEMENT == type) {
+      // If our parent is optional and there is an unexpected tag then return and
+      // we stay unbound.
+      if ((flavor & XER_OPTIONAL) && !check_name((const char*)reader.LocalName(), p_td, exer)) {
+        return -1;
+      }
       verify_name(reader, p_td, exer);
       depth = reader.Depth();
       if (reader.IsEmptyElement()) {
@@ -1538,6 +1547,19 @@ void OCTETSTRING_template::clean_up()
     else TTCN_error("Internal error: Invalid reference counter in an "
       "octetstring pattern.");
     break;
+  case DECODE_MATCH:
+    if (dec_match->ref_count > 1) {
+      dec_match->ref_count--;
+    }
+    else if (dec_match->ref_count == 1) {
+      delete dec_match->instance;
+      delete dec_match;
+    }
+    else {
+      TTCN_error("Internal error: Invalid reference counter in a "
+        "decoded content match.");
+    }
+    break;
   default:
     break;
   }
@@ -1567,6 +1589,10 @@ void OCTETSTRING_template::copy_template
     pattern_value = other_value.pattern_value;
     pattern_value->ref_count++;
     break;
+  case DECODE_MATCH:
+    dec_match = other_value.dec_match;
+    dec_match->ref_count++;
+    break;
   default:
     TTCN_error("Copying an uninitialized/unsupported octetstring template.");
   }
@@ -1817,6 +1843,14 @@ boolean OCTETSTRING_template::match(const OCTETSTRING& other_value,
     return template_selection == COMPLEMENTED_LIST;
   case STRING_PATTERN:
     return match_pattern(pattern_value, other_value.val_ptr);
+  case DECODE_MATCH: {
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL, TTCN_EncDec::EB_WARNING);
+    TTCN_EncDec::clear_error();
+    TTCN_Buffer buff(other_value);
+    boolean ret_val = dec_match->instance->match(buff);
+    TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL,TTCN_EncDec::EB_DEFAULT);
+    TTCN_EncDec::clear_error();
+    return ret_val; }
   default:
     TTCN_error("Matching an uninitialized/unsupported octetstring template.");
   }
@@ -1892,12 +1926,15 @@ int OCTETSTRING_template::lengthof() const
 void OCTETSTRING_template::set_type(template_sel template_type,
   unsigned int list_length)
 {
-  if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST)
-    TTCN_error("Setting an invalid list type for an octetstring template.");
+  if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST &&
+      template_type != DECODE_MATCH)
+    TTCN_error("Setting an invalid type for an octetstring template.");
   clean_up();
   set_selection(template_type);
-  value_list.n_values = list_length;
-  value_list.list_value = new OCTETSTRING_template[list_length];
+  if (template_type != DECODE_MATCH) {
+    value_list.n_values = list_length;
+    value_list.list_value = new OCTETSTRING_template[list_length];
+  }
 }
 
 OCTETSTRING_template& OCTETSTRING_template::list_item(unsigned int list_index)
@@ -1910,6 +1947,35 @@ OCTETSTRING_template& OCTETSTRING_template::list_item(unsigned int list_index)
   return value_list.list_value[list_index];
 }
 
+void OCTETSTRING_template::set_decmatch(Dec_Match_Interface* new_instance)
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Setting the decoded content matching mechanism of a non-decmatch "
+      "octetstring template.");
+  }
+  dec_match = new decmatch_struct;
+  dec_match->ref_count = 1;
+  dec_match->instance = new_instance;
+}
+
+void* OCTETSTRING_template::get_decmatch_dec_res() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoding result of a non-decmatch octetstring "
+      "template.");
+  }
+  return dec_match->instance->get_dec_res();
+}
+
+const TTCN_Typedescriptor_t* OCTETSTRING_template::get_decmatch_type_descr() const
+{
+  if (template_selection != DECODE_MATCH) {
+    TTCN_error("Retrieving the decoded type's descriptor in a non-decmatch "
+      "octetstring template.");
+  }
+  return dec_match->instance->get_type_descr();
+}
+
 void OCTETSTRING_template::log() const
 {
   switch (template_selection) {
@@ -1938,6 +2004,10 @@ void OCTETSTRING_template::log() const
     }
     TTCN_Logger::log_event_str("'O");
     break;
+  case DECODE_MATCH:
+    TTCN_Logger::log_event_str("decmatch ");
+    dec_match->instance->log();
+    break;
   default:
     log_generic();
     break;
@@ -1964,9 +2034,11 @@ void OCTETSTRING_template::log_match(const OCTETSTRING& match_value,
 void OCTETSTRING_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE|Module_Param::BC_LIST, "octetstring template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -2016,6 +2088,7 @@ void OCTETSTRING_template::set_param(Module_Param& param) {
   }
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* OCTETSTRING_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -2054,6 +2127,9 @@ Module_Param* OCTETSTRING_template::get_param(Module_Param_Name& param_name) con
       sizeof(unsigned short));
     mp = new Module_Param_Octetstring_Template(pattern_value->n_elements, val_cpy);
     break; }
+  case DECODE_MATCH:
+    mp->error("Referencing a decoded content matching template is not supported.");
+    break;
   default:
     break;
   }
@@ -2063,6 +2139,7 @@ Module_Param* OCTETSTRING_template::get_param(Module_Param_Name& param_name) con
   mp->set_length_restriction(get_length_range());
   return mp;
 }
+#endif
 
 void OCTETSTRING_template::encode_text(Text_Buf& text_buf) const
 {
