@@ -57,6 +57,11 @@ bool Module_Param_Id::next_name(int offset) {
   return false;
 }
 
+size_t Module_Param_Id::get_nof_names() const {
+  TTCN_error("Internal error: Module_Param_Id::get_nof_names()");
+  return 0;
+}
+
 char* Module_Param_Name::get_str() const {
   char* result = NULL;
   for (size_t i = 0; i < names.size(); ++i) {
@@ -245,6 +250,11 @@ char* Module_Param::get_pattern() const {
   return NULL;
 }
 
+boolean Module_Param::get_nocase() const {
+  TTCN_error("Internal error: Module_Param::get_nocase()");
+  return FALSE;
+}
+
 verdicttype Module_Param::get_verdict() const {
   TTCN_error("Internal error: Module_Param::get_verdict()");
   return NONE;
@@ -255,10 +265,12 @@ char* Module_Param::get_enumerated() const {
   return NULL;
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param_Ptr Module_Param::get_referenced_param() const {
   TTCN_error("Internal error: Module_Param::get_referenced_param()");
   return NULL;
 }
+#endif
 
 Module_Param::expression_operand_t Module_Param::get_expr_type() const { 
   TTCN_error("Internal error: Module_Param::get_expr_type()");
@@ -312,6 +324,7 @@ Module_Param_Ptr& Module_Param_Ptr::operator=(const Module_Param_Ptr& r) {
   return *this;
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param_Reference::Module_Param_Reference(Module_Param_Name* p): mp_ref(p) {
   if (mp_ref == NULL) {
     TTCN_error("Internal error: Module_Param_Reference::Module_Param_Reference()");
@@ -323,13 +336,13 @@ Module_Param_Ptr Module_Param_Reference::get_referenced_param() const {
     error("References to other variables are not allowed.");
   }
   mp_ref->reset();
-  Module_Param_Ptr ptr = Module_List::get_param(*mp_ref);
+  Module_Param_Ptr ptr = Module_List::get_param(*mp_ref, this);
   ptr.set_temporary();
   return ptr;
 }
 
 char* Module_Param_Reference::get_enumerated() const {
-  if (mp_ref->is_single_name()) {
+  if (mp_ref->get_nof_names() == (size_t)1) {
     return mp_ref->get_current_name();
   }
   return NULL;
@@ -342,6 +355,7 @@ void Module_Param_Reference::log_value() const {
 void Module_Param_Unbound::log_value() const {
   TTCN_Logger::log_event_str("<unbound>");
 }
+#endif
 
 Module_Param_Expression::Module_Param_Expression(expression_operand_t p_type,
   Module_Param* p_op1, Module_Param* p_op2)
@@ -542,7 +556,11 @@ void Module_Param_StringRange::log_value() const {
 }
 
 void Module_Param_Pattern::log_value() const {
-  TTCN_Logger::log_event_str("pattern \"");
+  TTCN_Logger::log_event_str("pattern ");
+  if (nocase) {
+    TTCN_Logger::log_event_str("@nocase ");
+  }
+  TTCN_Logger::log_event_str("\"");
   TTCN_Logger::log_event_str(pattern);
   TTCN_Logger::log_event_str("\"");
 }
@@ -716,11 +734,23 @@ void Module_Param::type_error(const char* expected, const char* type_name /* = N
     }
     // either use this parameter's or the referenced parameter's type string
     // (but never the head's type string)
-    reporter->error("Type mismatch: %s or reference to %s was expected%s%s instead of %s%s.",
-      expected, expected,
+    reporter->error("Type mismatch: %s "
+#ifdef TITAN_RUNTIME_2
+    "or reference to %s "
+#endif
+    "was expected%s%s instead of %s%s.",
+      expected,
+#ifdef TITAN_RUNTIME_2
+      expected,
+#endif
       (type_name != NULL) ? " for type " : "", (type_name != NULL) ? type_name : "",
+#ifdef TITAN_RUNTIME_2
       (get_type() == MP_Reference) ? "reference to " : "",
-      (get_type() == MP_Reference) ? get_referenced_param()->get_type_str() : get_type_str());
+      (get_type() == MP_Reference) ? get_referenced_param()->get_type_str() : get_type_str()
+#else
+      "", get_type_str()
+#endif
+      );
   }
 }
 
