@@ -111,15 +111,18 @@ void VERDICTTYPE::log() const
 void VERDICTTYPE::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_VALUE, "verdict value");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   if (mp->get_type()!=Module_Param::MP_Verdict) param.type_error("verdict value");
   const verdicttype verdict = mp->get_verdict();
   if (!IS_VALID(verdict)) param.error("Internal error: invalid verdict value (%d).", verdict);
   verdict_value = verdict;
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* VERDICTTYPE::get_param(Module_Param_Name& /* param_name */) const
 {
   if (!is_bound()) {
@@ -127,6 +130,7 @@ Module_Param* VERDICTTYPE::get_param(Module_Param_Name& /* param_name */) const
   }
   return new Module_Param_Verdict(verdict_value);
 }
+#endif
 
 void VERDICTTYPE::encode_text(Text_Buf& text_buf) const
 {
@@ -328,6 +332,12 @@ int VERDICTTYPE::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& p_reader
     if (name_tag)      for (; rd_ok == 1; rd_ok = p_reader.Read()) {
         type = p_reader.NodeType();
         if (XML_READER_TYPE_ELEMENT == type) {
+          // If our parent is optional and there is an unexpected tag then return and
+          // we stay unbound.
+          if ((p_flavor & XER_OPTIONAL) && !check_name((const char*)p_reader.LocalName(), p_td, e_xer)) {
+            return -1;
+          }
+          verify_name(p_reader, p_td, e_xer);
           rd_ok = p_reader.Read();
           break;
         }
@@ -682,9 +692,11 @@ void VERDICTTYPE_template::log_match(const VERDICTTYPE& match_value,
 void VERDICTTYPE_template::set_param(Module_Param& param) {
   param.basic_check(Module_Param::BC_TEMPLATE, "verdict template");
   Module_Param_Ptr mp = &param;
+#ifdef TITAN_RUNTIME_2
   if (param.get_type() == Module_Param::MP_Reference) {
     mp = param.get_referenced_param();
   }
+#endif
   switch (mp->get_type()) {
   case Module_Param::MP_Omit:
     *this = OMIT_VALUE;
@@ -714,6 +726,7 @@ void VERDICTTYPE_template::set_param(Module_Param& param) {
   is_ifpresent = param.get_ifpresent() || mp->get_ifpresent();
 }
 
+#ifdef TITAN_RUNTIME_2
 Module_Param* VERDICTTYPE_template::get_param(Module_Param_Name& param_name) const
 {
   Module_Param* mp = NULL;
@@ -753,6 +766,7 @@ Module_Param* VERDICTTYPE_template::get_param(Module_Param_Name& param_name) con
   }
   return mp;
 }
+#endif
 
 void VERDICTTYPE_template::encode_text(Text_Buf& text_buf) const
 {
