@@ -52,17 +52,17 @@ extern bool t_flag_used;
 void XSDName2TTCN3Name(const Mstring& in, QualifiedNames & used_names, modeType type_of_the_name,
   Mstring & res, Mstring & variant, bool no_replace) {
   static const char* TTCN3_reserved_words[] = {
-    "action", "activate", "address", "alive", "all", "alt", "altstep", "and", "and4b", "any", "anytype", "apply",
+    "action", "activate", "address", "alive", "all", "alt", "altstep", "and", "and4b", "any", "any2unistr", "anytype", "apply",
     "bitstring", "boolean", "break",
     "call", "case", "catch", "char", "charstring", "check", "clear", "complement", "component", "connect",
     "const", "continue", "control", "create",
-    "deactivate", "default", "derefers", "disconnect", "display", "do", "done",
-    "else", "encode", "enumerated", "error", "except", "exception", "execute", "extends", "extension",
+    "decmatch", "deactivate", "decvalue_unichar", "default", "derefers", "disconnect", "display", "do", "done",
+    "else", "encode", "encvalue_unichar", "enumerated", "error", "except", "exception", "execute", "extends", "extension",
     "external",
     "fail", "false", "float", "for", "friend", "from", "function",
     "getverdict", "getcall", "getreply", "goto", "group",
-    "halt", "hexstring",
-    "if", "if present", "import", "in", "inconc", "infinity", "inout", "integer", "interleave",
+    "halt", "hexstring", "hostid",
+    "if", "if present", "import", "in", "inconc", "infinity", "inout", "integer", "interleave", "istemplatekind",
     "kill", "killed",
     "label", "language", "length", "log",
     "map", "match", "message", "mixed", "mod", "modifies", "module", "modulepar", "mtc",
@@ -519,7 +519,7 @@ bool matchDates(const char * string, const char * type) {
   const Mstring second("([0-5][0-9])");
   const Mstring endofdayext("24:00:00(.0?)?");
   const Mstring yearext("((-)([1-9][0-9]*)?)?");
-  const Mstring timezone("(Z|[+-]((0[0-9]|1[0-3]):[0-5][0-9]|14:00))?");
+  const Mstring time_zone("(Z|[+-]((0[0-9]|1[0-3]):[0-5][0-9]|14:00))?");
   const Mstring fraction("(.[0-9]+)?");
   const Mstring nums("[0-9]+");
   const Mstring durtime("(T[0-9]+"
@@ -528,24 +528,24 @@ bool matchDates(const char * string, const char * type) {
 
   Mstring pattern;
   if (strcmp(type, "gDay") == 0) {
-    pattern = Mstring("(---)") + day + timezone;
+    pattern = Mstring("(---)") + day + time_zone;
   } else if (strcmp(type, "gMonth") == 0) {
-    pattern = Mstring("(--)") + month + timezone;
+    pattern = Mstring("(--)") + month + time_zone;
   } else if (strcmp(type, "gYear") == 0) {
-    pattern = yearext + year + timezone;
+    pattern = yearext + year + time_zone;
   } else if (strcmp(type, "gYearMonth") == 0) {
-    pattern = yearext + year + Mstring("(-)") + month + timezone;
+    pattern = yearext + year + Mstring("(-)") + month + time_zone;
   } else if (strcmp(type, "gMonthDay") == 0) {
-    pattern = Mstring("(--)") + month + Mstring("(-)") + day + timezone;
+    pattern = Mstring("(--)") + month + Mstring("(-)") + day + time_zone;
   } else if (strcmp(type, "date") == 0) {
-    pattern = yearext + year + Mstring("(-)") + month + Mstring("(-)") + day + timezone;
+    pattern = yearext + year + Mstring("(-)") + month + Mstring("(-)") + day + time_zone;
   } else if (strcmp(type, "time") == 0) {
     pattern = Mstring("(") + hour + Mstring(":") + minute + Mstring(":") + second +
-      fraction + Mstring("|") + endofdayext + Mstring(")") + timezone;
+      fraction + Mstring("|") + endofdayext + Mstring(")") + time_zone;
   } else if (strcmp(type, "dateTime") == 0) {
     pattern = yearext + year + Mstring("(-)") + month + Mstring("(-)") + day +
       Mstring("T(") + hour + Mstring(":") + minute + Mstring(":") + second +
-      fraction + Mstring("|") + endofdayext + Mstring(")") + timezone;
+      fraction + Mstring("|") + endofdayext + Mstring(")") + time_zone;
   } else if (strcmp(type, "duration") == 0) {
     pattern = Mstring("(-)?P(") + nums + Mstring("(Y(") + nums + Mstring("(M(") +
       nums + Mstring("D") + durtime + Mstring("?|") + durtime + Mstring("?|D") +
@@ -729,7 +729,10 @@ int multi(const TTCN3Module *module, ReferenceData const& outside_reference,
   RootType * ct = ::lookup1(module, outside_reference.get_val(), outside_reference.get_uri(), obj, want_CT);
   if (st || ct) {
     multiplicity = 1; // locally defined, no qualif needed
-  } else for (List<const TTCN3Module*>::iterator it = module->getImportedModules().begin(); it; it = it->Next) {
+    // means that outside_reference.get_uri() == module->getTargetNamespace())
+  } else {
+    // Look for definitions in the imported modules
+    for (List<const TTCN3Module*>::iterator it = module->getImportedModules().begin(); it; it = it->Next) {
       // Artificial lookup
       st = ::lookup1(it->Data, outside_reference.get_val(), it->Data->getTargetNamespace(), obj, want_ST);
       ct = ::lookup1(it->Data, outside_reference.get_val(), it->Data->getTargetNamespace(), obj, want_CT);
@@ -737,6 +740,17 @@ int multi(const TTCN3Module *module, ReferenceData const& outside_reference,
         ++multiplicity;
       }
     }
+    // If multiplicity > 1 then the qualif needed
+    // But if == 1 we need to check this module for a type definition with
+    // the same name as outsize_reference.get_val()
+    if (multiplicity == 1) {
+      st = ::lookup1(module, outside_reference.get_val(), module->getTargetNamespace(), obj, want_ST);
+      ct = ::lookup1(module, outside_reference.get_val(), module->getTargetNamespace(), obj, want_CT);
+      if (st || ct) {
+        ++multiplicity;
+      }
+    }
+  }
   return multiplicity;
 }
 
