@@ -227,7 +227,7 @@ void SimpleType::loadWithValues() {
       value.modified = true;
       break;
     case n_fractionDigits:
-      //addVariant(V_fractionDigits, atts.value);
+      addVariant(V_fractionDigits, atts.value);
       break;
     case n_label:
       addComment(Mstring("LABEL:"));
@@ -488,10 +488,19 @@ void SimpleType::setReference(const Mstring& ref, bool only_name_dependency) {
       name.upload(ref);
     }
     if (type.convertedValue.empty() || type.convertedValue == "anySimpleType") {
-      type.upload(ref.getValueWithoutPrefix(':'));
+      type.upload(ref);
+    }
+    bool found = false;
+    for (List<NamespaceType>::iterator ns = getModule()->getDeclaredNamespaces().begin(); ns; ns = ns->Next) {
+      if (ns->Data.uri != XMLSchema && type.refPrefix == ns->Data.prefix.c_str()) {
+        found = true;
+        break;
+      }
     }
     fromRef = true;
-    return;
+    if (!found) {
+      return;
+    }
   }
 
   Mstring refPrefix = ref.getPrefix(':');
@@ -547,7 +556,7 @@ void SimpleType::referenceResolving() {
   }
   if(outside_reference.empty() && substitutionGroup.empty()) return;
   if (outside_reference.is_resolved()) return;
-
+  
   if(!outside_reference.empty()){
     SimpleType * found_ST = static_cast<SimpleType*> (
       TTCN3ModuleInventory::getInstance().lookup(this, want_ST));
@@ -587,20 +596,19 @@ void SimpleType::referenceResolving() {
 
 void SimpleType::referenceForST(SimpleType * found_ST) {
   outside_reference.set_resolved(found_ST);
-
   if (in_name_only)
     return;
 
+  if (!found_ST->builtInBase.empty()) {
+    builtInBase = found_ST->builtInBase;
+  }
+
   if (construct == c_element)
     return;
 
   if (mode == listMode || mode == restrictionAfterListMode)
     return;
 
-  if (!found_ST->builtInBase.empty()) {
-    builtInBase = found_ST->builtInBase;
-  }
-
   length.applyReference(found_ST->length);
   pattern.applyReference(found_ST->pattern);
   enumeration.applyReference(found_ST->enumeration);
@@ -775,18 +783,6 @@ void SimpleType::finalModification() {
   }
 
   isOptional = isOptional || (getMinOccurs() == 0 && getMaxOccurs() == 0);
-
-  // If the type name is the same as the identifier then we have to prefix it 
-  // with the module identifier.
-  if (type.convertedValue == name.convertedValue && !outside_reference.empty()) {
-    List<const TTCN3Module*>::iterator import_module = module->getImportedModules().begin();
-    for (; import_module; import_module = import_module->Next) {
-      if (import_module->Data->getTargetNamespace() == outside_reference.get_uri()) {
-        type.upload(import_module->Data->getModulename() + Mstring(".") + type.convertedValue);
-        break;
-      }
-    }
-  }
 }
 
 bool SimpleType::hasUnresolvedReference() {
@@ -797,6 +793,10 @@ bool SimpleType::hasUnresolvedReference() {
   }
 }
 
+void SimpleType::modifyList() {
+  // Intentional empty
+}
+
 void SimpleType::applyRefAttribute(const Mstring& ref_value) {
   if (!ref_value.empty()) {
     setReference(ref_value);
@@ -835,8 +835,8 @@ void SimpleType::printToFile(FILE * file) {
 
     int multiplicity = multi(module, outside_reference, this);
     const RootType *type_ref = outside_reference.get_ref();
-    if ((multiplicity > 1) && type_ref
-      && type_ref->getModule() != module) {
+    if ((multiplicity > 1 && type_ref && type_ref->getModule() != module)
+       || name.convertedValue == type.convertedValue) {
       fprintf(file, "%s.", type_ref->getModule()->getModulename().c_str());
     }
 
