@@ -34,6 +34,7 @@
 #define LLONG_MAX 9223372036854775807LL
 #endif
 
+extern bool n_flag_used;
 extern bool V_flag_used;
 extern bool w_flag_used;
 extern bool x_flag_used;
@@ -54,7 +55,8 @@ XMLParser::XMLParser(const char * a_filename)
 , actualTagName(n_NOTSET)
 , actualTagAttributes(this)
 , parentTagNames()
-, inside_annotation(){
+, inside_annotation()
+, lastWasListEnd(false){
   xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
 
   parserCheckingXML = (xmlSAXHandler *) malloc(sizeof (xmlSAXHandler));
@@ -62,6 +64,9 @@ XMLParser::XMLParser(const char * a_filename)
   parserCheckingXML->initialized = XML_SAX2_MAGIC;
   parserCheckingXML->warning = warningHandler;
   parserCheckingXML->error = errorHandler;
+  if (n_flag_used) {
+    parserCheckingXML->startElementNs = (startElementNsSAX2Func) wrapper_to_call_startelement_when_xsd_read_h;
+  }
   contextCheckingXML = xmlCreateFileParserCtxt(a_filename);
   if (!contextCheckingXML) {
     fprintf(stderr,
@@ -71,6 +76,9 @@ XMLParser::XMLParser(const char * a_filename)
     return;
   }
   contextCheckingXML->sax = parserCheckingXML;
+  if (n_flag_used) {
+    contextCheckingXML->userData = this;
+  }
 
   if (!x_flag_used) {
     contextCheckingXSD = xmlSchemaNewParserCtxt(a_filename);
@@ -126,7 +134,7 @@ void XMLParser::checkSyntax() {
 }
 
 void XMLParser::validate() {
-  if (!x_flag_used) {
+  if (!x_flag_used && !n_flag_used) {
     xmlSchemaPtr schema = xmlSchemaParse(contextCheckingXSD);
     if (schema) {
       xmlSchemaValidCtxtPtr validator = xmlSchemaNewValidCtxt(schema);
@@ -170,6 +178,11 @@ void XMLParser::wrapper_to_call_characterdata_h(XMLParser *self, const xmlChar *
   self->characterdataHandler(ch, len);
 }
 
+void XMLParser::wrapper_to_call_startelement_when_xsd_read_h(XMLParser *self, const xmlChar * localname, const xmlChar *, const xmlChar *,
+  int nb_namespaces, const xmlChar ** namespaces, const int nb_attributes, int, const xmlChar ** attributes) {
+  self->startelementHandlerWhenXSDRead(localname, nb_namespaces, namespaces, nb_attributes, attributes);
+}
+
 void XMLParser::warningHandler(void *, const char *, ...) {
   if (w_flag_used) {
     return;
@@ -259,6 +272,10 @@ void XMLParser::startelementHandler(const xmlChar * localname,
           module->loadValuesFromSchemaTag(actualTagAttributes.targetNamespace, declaredNamespaces,
             actualTagAttributes.elementFormDefault, actualTagAttributes.attributeFormDefault,
             actualTagAttributes.blockDefault);
+          // Only need the targetNamespace. Stop.
+          if (n_flag_used) {
+            xmlStopParser(context);
+          }
           break;
         }
         default:
@@ -302,9 +319,13 @@ void XMLParser::startelementHandler(const xmlChar * localname,
 
     default:
       if (module->hasDefinedMainType()) {
-         if(actualTagName == n_annotation ||
-            actualTagName == n_appinfo ||
-            actualTagName == n_documentation){
+        if (lastWasListEnd) {
+          ((SimpleType&)(module->getLastMainType())).modifyList();
+          lastWasListEnd = false;
+        }
+        if(actualTagName == n_annotation ||
+          actualTagName == n_appinfo ||
+          actualTagName == n_documentation){
           inside_annotation.push_back(actualTagName);
           module->getLastMainType().loadWithValues();
         }else if(inside_annotation.empty()){
@@ -323,6 +344,17 @@ void XMLParser::startelementHandler(const xmlChar * localname,
 
   ++actualDepth;
   parentTagNames.push_back(actualTagName);
+  if (lastWasListEnd) {
+    lastWasListEnd = false;
+  }
+}
+
+void XMLParser::startelementHandlerWhenXSDRead(const xmlChar * localname,
+  int nb_namespaces, const xmlChar ** namespaces, int nb_attributes, const xmlChar ** attributes) {
+  if (strcmp((const char*)localname, "schema") == 0) {
+    // Don't parse beyond the schema tag.
+    xmlStopParser(contextCheckingXML);
+  }
 }
 
 void XMLParser::endelementHandler(const xmlChar * localname) {
@@ -355,12 +387,15 @@ void XMLParser::endelementHandler(const xmlChar * localname) {
   }
   
   if(tag == n_list) {
-      if(module->hasDefinedMainType()) {
-          SimpleType& st = (SimpleType&)(module->getLastMainType());
-          if(st.getXsdtype() == n_NOTSET){
-            st.setMode(SimpleType::restrictionAfterListMode);
-          }
+    lastWasListEnd = true;
+    if(module->hasDefinedMainType()) {
+      SimpleType& st = (SimpleType&)(module->getLastMainType());
+      if(st.getXsdtype() == n_NOTSET){
+        st.setMode(SimpleType::restrictionAfterListMode);
       }
+    }
+  } else if (tag != n_simpleType){
+    lastWasListEnd = false;
   }
 
   --actualDepth;
@@ -469,11 +504,6 @@ void XMLParser::fillUpActualTagName(const char * localname, const tagMode mode)
     }
   } else if (name_s == "fractionDigits") {
     actualTagName = n_fractionDigits;
-    if (mode == startElement) {
-      printWarning(filename, xmlSAX2GetLineNumber(context),
-        Mstring("The 'fractionDigits' tag is currently not supported."));
-      ++num_warnings;
-    }
   } else if (name_s == "group")
     actualTagName = n_group;
   else if (name_s == "import")
